#!/bin/bash
# Copyright 2007 Haydrian Corporation
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# Bryan McLellan <btm@loftninjas.org>

# Important notes about building deb 2.0 deb files
# A deb is an ar archive containing three files in the following order
# - debian-binary (contains one line, 2.0, specifing the deb version)
# - control.tar.gz (contents of the DEBIAN folder, scripts, control, etc)
# - data.tar.gz (contains the actual files to be installed)
# It is important to note that this file must be in the BSD format, which
# does not support spaces in filenames. The GNU format does, and terminates
# filenames with a /, which breaks certain apt utilities. head -2 a deb or
# ar file to see if the / comes after the filename. The implication of this
# is that you can not use binutils/gnu ar in a simple manner for making debs

CWD=`pwd`
DATA="./apps/oracle ./etc/oratab ./etc/oraInst.loc ./usr/local/bin/coraenv ./usr/local/bin/dbhome ./usr/local/bin/oraenv"

if [ `id -u` -ne 0 ]; then
	echo "must run as root"
	exit
fi

if [ "$1" = "" ] && [ "$1" != "build" ] && [ "$1" != "control" ]; then
	echo "You should have already modified DEBIAN/control"
	echo .
	echo "$0 build	-- full build (control && data) (also cleans)"
	echo "$0 control  -- recopies DEBIAN and rebuilds"
	echo "$0 clean -- clean up so we can start over"
	exit
fi

if [ "$1" = "clean" ] || [ "$1" = "build" ]; then
	echo "Cleaning"
	rm -rf build
	exit
fi

if [ "$1" = "build" ]; then
	echo "Building Tree"
	mkdir build
	cd /
	tar -c $DATA | tar -xC $CWD/build
	cd $CWD
fi

echo "Creating postinst"
cat oradeb.py > DEBIAN/postinst
cat oradeb.py.postinst >> DEBIAN/postinst

echo "Copying in DEBIAN Files"
cp -R DEBIAN build

chown -R root:root build/DEBIAN

echo "Creating deb"
dpkg-deb -b build .

