#!/usr/bin/env python
# Copyright 2007 Haydrian Corporation
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# Bryan McLellan -- btm@loftninjas.org

import sys,commands,os
import oradeb

ora_base = oradeb.ora_base	# where we'll stick the user and other stuff
ora_home = oradeb.ora_home	# where the software goes
ora_inv = oradeb.ora_inv	# where oracle keeps track of junk
user = oradeb.user
password = oradeb.password

installurl = "http://yourserver.local/oracle" # FIXME this must be set to a place where the below large file exists
installfile = "10201_database_linux_x86_64.cpio.gz"
responsefile = "custom.rsp"
memneeded = 512000 # KB of ram needed
freetmp = 400000 # KB free neeeded in /tmp
freedisk = 2000000 # KB free needed in /

# packages required. note that ia32-libs and libc6-dev-i386 are required on x86_64
requiredpackages = ['bash', 'build-essential', 'libaio1', 'lesstif2', 'ia32-libs', 'rpm', 'libc6-dev-i386'] 

### check to be root
if commands.getoutput('id -u') != "0":
  print "must be root to execute this script"
  sys.exit()

### check for installation files ###
if not os.path.isfile(installfile):
	print 'Install file not found:', installfile
	os.system("wget %s/%s" % (installurl, installfile))

if not os.path.isfile(responsefile):
  print 'Response file not found:', responsefile 
  sys.exit()

installbase = os.getcwd() 	# save where we are for later

# check for necessary RAM and swap
meminfo = open("/proc/meminfo", "r")
for memline in meminfo.readlines():
  memline = memline.replace('=','')
  memline = memline.replace(':','')
  memcols = memline.split()

  if memcols[0] == 'MemTotal':
    print "Checking ram...",
    memtotal = int(memcols[1])
    print memtotal, "KB",
    if memtotal < memneeded:
      print "insufficient ---", memneeded, "KB required"
      sys.exit()
    else:
      print "OK"

    # calculate swap needed based on ram per Oracle
    if memtotal <= 1024000:
      swapneeded = memtotal * 2
    elif memtotal > 1024000 and memtotal <= 2048000:
      swapneeded = memtotal * 1.5
    elif memtotal > 2048000 and memtotal <= 8192000:
      swapneeded = memtotal
    elif memcols[1] > 8192000:
      swapneeded = memtotal * 0.75
    continue
  
  if memcols[0] == 'SwapTotal':
    print "Checking swap...",
    swaptotal = int(memcols[1])
    print swaptotal, "KB",
    if swaptotal < swapneeded:
      print "insufficient ---", swapneeded, "KB required"  
      sys.exit()
    else:
      print "OK"
    break

# check for free disk space, df with a path should return only one mount
# -P posix compatibility, sometimes we get multiple rows per mount otherwise
print "Checking freespace in /tmp...",
diskfreetmp = commands.getoutput('df -P -k /tmp')
for line in diskfreetmp.split('\n'):
  linecols = line.split()
  if linecols[3].isdigit():
    freespace = linecols[3]
  else:
    continue
 
print freespace, "KB", 
if freespace < freetmp:
  print "insufficient ---", freetmp, "KB required"
  sys.exit()
else:
  print "OK"

print "Checking freespace in /...",
diskfree = commands.getoutput('df -P -k /')
for line in diskfree.split('\n'):
  linecols = line.split()
  if linecols[3].isdigit():
    freespace = linecols[3]
  else:
    continue

print freespace, "KB",
if freespace < freedisk:
  print "insufficient ---", freedisk, "KB required"
  sys.exit()
else:
  print "OK"

### Check for FQDN
print "Checking hostname...",
fqdn = commands.getoutput('hostname -f')
print fqdn,
if fqdn.count('.') > 2:
  print "appears valid"
else:
  print "appears invalid, check 'hostname', 'hostname -f', /etc/nsswitch.conf and /etc/hosts"
  sys.exit()


### Checking for required packages ###
print "Checking packages:"
missingpackage = []
for package in requiredpackages:
  if commands.getstatusoutput("dpkg -s " + package + " | grep 'Status: install ok installed'")[0] > 0:
    print "\t" + package, "not installed"
    missingpackage.append(package)
  else:
    print "\t" + package, "installed"

### Install Required packages ###
if missingpackage:
  print "Installing missing packages...",
  for package in missingpackage:
    installpackage = commands.getstatusoutput("apt-get -y install " + package)
    if installpackage[0] > 0:
      print package, "failed..."
      print installpackage[1]
      sys.exit()
    else:
      print package,

print "Preparing users and groups...",
oradeb.user_prep(user, password, ora_base)
print "done"

### sysctl ###
print "Adding (but not checking) values to /etc/sysctl.conf..."
oradeb.add_sysctl()

print "Reloading sysctl values..."
commands.getstatusoutput('sysctl -p')

### limits ###
# FIXME : Doesn't modify existing variables or warn
print "Adding (but not checking) values to /etc/security/limits.conf..."
oradeb.add_limit()

print "Preparing directory tree..."
oradeb.build_home(ora_base, ora_home, user)

### hacks ###
print "Performing foolish filesystem hacks..."
oradeb.create_links()

### fix dash/bash ###
print "Disabling dash as /bin/sh..."
oradeb.config_dash()

### Creating install files ###
print "Creating inventory control file..."
oradeb.create_orainv(ora_inv)

# FIXME : assume answer file is in ~
# FIXME : assume install is uncompressed in ~/database
print "Extracting install files...",
os.chdir(ora_base)
extractcpio = commands.getstatusoutput("gunzip -c %s/%s | cpio -idmv" % (installbase,installfile ))
if extractcpio[0] > 0:
  print "failed"
  print extractcpio[1]
  sys.exit(1)
print ".done"
 
os.chdir('database')
os.system("cp %s/%s ." % (installbase, responsefile))

### tell us what to do
# FIXME, no way to deal with variables. must set, then run installer.
os.environ["ORACLE_HOSTNAME"] = fqdn

### run installer (runinstaller spawns oui, how to track pids of children?)
#os.system("su oracle -c 'cd ~/database ; ./runInstaller -IgnoreSysPrereqs -silent -responseFile %s/database/custom.rsp'" % (ora_base))

