#!/usr/bin/env python
# Copyright 2007 Haydrian Corporation
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Common functions for Haydrian oracle installation and packaging scripts
# -- Bryan McLellan <btm@loftninjas.org

import pwd,grp,commands,os,sys

ora_base = "/apps/oracle"   # where we'll stick the user and other stuff
ora_home = "/apps/oracle/10g"   # where the software goes
ora_inv = "/apps/oracle/oraInventory"   # where oracle keeps track of junk
user = "oracle"
password = "" #FIXME this must be set

sysctl = [('kernel.shmall', '2097152'), ('kernel.shmmax', '2147483648'),
    ('kernel.shmmni', '4096'), ('kernel.sem', '250 32000 100 128'),
    ('net.ipv4.ip_local_port_range', '1024 65000'), ('net.core.rmem_default', '262144'),
    ('net.core.rmem_max', '262144'), ('net.core.wmem_default', '262144'),
    ('net.core.wmem_max', '262144'), ('vm.swappiness', '10')]
limit = [('* soft nproc', '2047'), ('* hard nproc', '16384'),
    ('* soft nofile', '1024'), ('* hard nofile', '65536')]

def add_group(group):
	"""Adds the specified group to the system"""
	
	try:
		grp.getgrnam(group)
	except KeyError:
		if commands.getstatusoutput("groupadd %s" % (group))[0] != 0:
			print "Unable to add group: %s" (group)
			sys.exit(1)
	finally:
		return grp.getgrnam(group)

def user_prep(user,password,ora_base):
	"""Performs user and group system preparation for oracle. """
	
	# Debian uses nogroup, so we add nobody for oracle per documentation
	add_group('nobody')
	add_group('dba')
	add_group('oinstall')

	try:
		pwd.getpwnam(user)
	except KeyError:
		commands.getstatusoutput("useradd -d %s -s /bin/bash -g oinstall -G dba %s" % (ora_base,user))
	
	# Set the users password from encrypted string
	if commands.getstatusoutput("usermod -p %s %s" % (password,user))[0] != 0:
		print "Error setting password for $user"
		sys.exit(1)
	

def add_sysctl():
	"""Configures oracle related values in /etc/sysctl.conf."""

	#FIXME: Should warn on existing values
	sysctlconf = open('/etc/sysctl.conf', 'a')
	for sysctlpair in sysctl:
		if commands.getstatusoutput('grep ' + sysctlpair[0] + ' /etc/sysctl.conf')[0] > 0:
			sysctlconf.write("%s = %s\n" % (sysctlpair[0],sysctlpair[1]))
	sysctlconf.close()

def add_limit():
	"""Configures oracle related values in /etc/security/limits.conf"""

	limitconf = open('/etc/security/limits.conf', 'a')
	#limitconf.write("# Additions for Oracle")
	for limitpair in limit:
		if commands.getstatusoutput('grep "' + limitpair[0] + '" /etc/security/limits.conf')[0] > 0:
			limitconf.write(limitpair[0] + '\t' + limitpair[1] + '\n')

	limitconf.close()

def build_home(ora_base, ora_home, user):
	""" Creates user home tree. """
	os.system("mkdir -p %s %s" % (ora_base, ora_home))
	os.system("chown -R %s:oinstall %s" % (user,ora_base))
	os.system("chmod -R 755 %s" % (ora_base))

	# FIXME : unsafe
	oraclebashrc = open("%s/.bashrc" % (ora_base), 'w')
	oraclebashrc.write('umask 022\n')
	oraclebashrc.write("export ORACLE_HOME=%s" % (ora_home))
	oraclebashrc.close()
	os.system("chmod 644 %s/.bashrc" % (ora_base))
	os.system("chown %s:oinstall %s/.bashrc" % (user, ora_base))

	oraclebash_profile = open("%s/.bash_profile" % (ora_base), 'w')
	oraclebash_profile.write('umask 022\n')
	oraclebash_profile.write("export ORACLE_HOME=%s" % (ora_home))
	oraclebash_profile.close()
	os.system("chmod 644 %s/.bash_profile" % (ora_base))
	os.system("chown %s:oinstall %s/.bash_profile" % (user, ora_base))

def remove_home(ora_base):
	""" Cleans up after users home."""

	os.system("rm %s/.bashrc" % (ora_base))
	os.system("rm %s/.bash_profile" % (ora_base))
	os.system("rmdir -p %s" % (ora_base))

def create_links():
	"""Creates dirty oracle related links. """

	links = [('/bin/awk', '/usr/bin/awk'), ('/bin/rpm', '/usr/bin/rpm'), ('/bin/basename','/usr/bin/basename'), ('/lib/libgcc_s.so', '/lib/libgcc_s.so.1')]

	for linkpair in links:
		if not os.path.exists(linkpair[0]):
			if os.path.exists(linkpair[1]):
				os.symlink(linkpair[1], linkpair[0])
			else:
				print "Unable to resolve %s" % (linkpair[0])
				sys.exit(1)
	
def remove_links():
	"""Removes custom oracle related links. """

	links = ['/bin/awk', '/bin/rpm', '/bin/basename', '/lib/libgcc_s.so']

	for link in links:
		if islink(link):
			unlink(link)

def config_dash():
	"""Configures dash to not be the default sh (Oracle scripts are not POSIX compatible). """

	os.system('echo set dash/sh false | debconf-communicate ; dpkg-reconfigure -f noninteractive dash')

def create_orainv(ora_inv):
	"""Creates Oracle Inventory configuration file. """

	orainst = open('/etc/oraInst.loc', 'w')
	orainst.write("inventory_loc=%s\n" % (ora_inv))
	orainst.write('inst_group=\n')
	orainst.close()

	os.system('chown oracle:oinstall /etc/oraInst.loc')
	os.system('chmod 664 /etc/oraInst.loc')

